searchData={"items":[{"type":"module","title":"Loe","doc":"Loe is a tiny Elixir library for chaining `{:ok, _}`/`{:error, _}` results and raw values with elegant macros and helpers.\n\nIt helps normalize, transform, and chain computations while handling both successful and error results in a consistent way.","ref":"Loe.html"},{"type":"module","title":"Functions - Loe","doc":"- `lift/2` — Applies a function to a value if it is raw or `{:ok, _}`. Leaves `{:error, _}` or `:error` unchanged. Always returns `{:ok, _}` or `{:error, _}`.\n- `tfil/2` — The inverse of `lift/2`. Applies a function **only** if the input is `:error` or `{:error, _}`. Leaves successful values unchanged.\n- `unwrap!/1` — Extracts the value from `{:ok, value}`. Raises if the value is `:error` or `{:error, _}`.","ref":"Loe.html#module-functions"},{"type":"module","title":"Operators - Loe","doc":"- `~>>` — Infix form of `lift/2`, for chaining operations on success.\n- `<~>` — Infix form of `tfil/2`, for transforming errors.","ref":"Loe.html#module-operators"},{"type":"module","title":"Example - Loe","doc":"```\nvalue\n~>> validate()\n~>> normalize()\n<~> wrap_error()\n```","ref":"Loe.html#module-example"},{"type":"macro","title":"Loe.<~>/2","doc":"Infix `tfil`.\n\nAllows chaining like `value <~> fun()`.","ref":"Loe.html#%3C~%3E/2"},{"type":"macro","title":"Examples - Loe.<~>/2","doc":"iex> import Loe\n    ...> 5 <~> (fn x -> x + 2 end).()\n    {:ok, 5}\n\n    iex> import Loe\n    ...> {:error, 10} <~> (fn x -> x * 2 end).()\n    {:error, 20}\n\n    iex> import Loe\n    ...> {:error, 10} <~> (fn x -> {:ok, x * 2} end).()\n    {:ok, 20}\n\n    iex> import Loe\n    ...> :error <~> (fn _x -> {:error, :nice} end).()\n    {:error, :nice}","ref":"Loe.html#%3C~%3E/2-examples"},{"type":"type","title":"Loe.input/2","doc":"Input may be a raw value, `{:ok, value}`, `{:error, error}`, or `:error`.","ref":"Loe.html#t:input/2"},{"type":"function","title":"Loe.lift/2","doc":"Applies a function to a raw or boxed input and returns a normalized result.","ref":"Loe.html#lift/2"},{"type":"function","title":"Accepted input: - Loe.lift/2","doc":"- A raw value `a`\n  - A tuple `{:ok, a}`\n  - An error tuple `{:error, e}`\n  - A bare `:error`","ref":"Loe.html#lift/2-accepted-input"},{"type":"function","title":"Function fun may return: - Loe.lift/2","doc":"- A raw value `b`\n  - A tuple `{:ok, b}`\n  - A tuple `{:error, e}`\n  - A bare `:error`","ref":"Loe.html#lift/2-function-fun-may-return"},{"type":"function","title":"Result: - Loe.lift/2","doc":"Always returns either `{:ok, b}` or `{:error, e}`.","ref":"Loe.html#lift/2-result"},{"type":"function","title":"Examples - Loe.lift/2","doc":"iex> Loe.lift(2, fn x -> x * 2 end)\n    {:ok, 4}\n\n    iex> Loe.lift({:ok, 2}, fn x -> {:ok, x * 3} end)\n    {:ok, 6}\n\n    iex> Loe.lift({:ok, 2}, fn x -> x * 2 end)\n    {:ok, 4}\n\n    iex> Loe.lift(2, fn _x -> :error end)\n    {:error, :error}\n\n    iex> Loe.lift(2, fn _x -> {:error, :bad} end)\n    {:error, :bad}\n\n    iex> Loe.lift(:error, fn _ -> 123 end)\n    {:error, :error}\n\n    iex> Loe.lift({:error, :bad}, fn _ -> 123 end)\n    {:error, :bad}","ref":"Loe.html#lift/2-examples"},{"type":"type","title":"Loe.result/2","doc":"Normalized result value: either `{:ok, value}` or `{:error, error}`.","ref":"Loe.html#t:result/2"},{"type":"function","title":"Loe.tfil/2","doc":"Applies a function to a raw or boxed error input and returns a normalized result.","ref":"Loe.html#tfil/2"},{"type":"function","title":"Accepted input: - Loe.tfil/2","doc":"- A raw value `a`\n  - A tuple `{:ok, a}`\n  - An error tuple `{:error, e}`\n  - A bare `:error`","ref":"Loe.html#tfil/2-accepted-input"},{"type":"function","title":"Function fun may return: - Loe.tfil/2","doc":"- A raw value `b`\n  - A tuple `{:ok, b}`\n  - A tuple `{:error, e}`\n  - A bare `:error`","ref":"Loe.html#tfil/2-function-fun-may-return"},{"type":"function","title":"Result: - Loe.tfil/2","doc":"Always returns either `{:ok, b}` or `{:error, e}`.","ref":"Loe.html#tfil/2-result"},{"type":"function","title":"Examples - Loe.tfil/2","doc":"iex> Loe.tfil(2, fn _x -> :ignore_and_wrap end)\n    {:ok, 2}\n\n    iex> Loe.tfil({:ok, 2}, fn _x -> :ignore end)\n    {:ok, 2}\n\n    iex> Loe.tfil(:error, fn x -> %{a: x, b: 300} end)\n    {:error, %{a: :error, b: 300}}\n\n    iex> Loe.tfil({:error, :bad}, fn x -> %{a: x, b: 300} end)\n    {:error, %{a: :bad, b: 300}}\n\n    iex> Loe.tfil(:error, fn _x -> {:error, :reason} end)\n    {:error, :reason}\n\n    iex> Loe.tfil({:error, :bad}, fn _x -> {:error, :too_bad} end)\n    {:error, :too_bad}","ref":"Loe.html#tfil/2-examples"},{"type":"function","title":"Loe.unwrap!/1","doc":"Unwraps a result value or raises an error.\n\n- If given `{:ok, value}`, returns `value`.\n- If given `{:error, reason}`, raises `RuntimeError` with the reason.\n- If given anything else, raises `ArgumentError`.","ref":"Loe.html#unwrap!/1"},{"type":"function","title":"Examples - Loe.unwrap!/1","doc":"iex> Loe.unwrap!({:ok, 42})\n    42\n\n    iex> Loe.unwrap!({:error, :fail})\n    ** (RuntimeError) Loe.unwrap!/1 encountered error: :fail\n\n    iex> Loe.unwrap!(123)\n    ** (ArgumentError) Loe.unwrap!/1 expected {:ok, _} or {:error, _}, got: 123","ref":"Loe.html#unwrap!/1-examples"},{"type":"macro","title":"Loe.~>>/2","doc":"Infix `lift`.\n\nAllows chaining like `value ~>> fun()`.","ref":"Loe.html#~%3E%3E/2"},{"type":"macro","title":"Examples - Loe.~>>/2","doc":"iex> import Loe\n    ...> 5 ~>> (fn x -> x + 2 end).()\n    {:ok, 7}\n\n    iex> import Loe\n    ...> {:ok, 10} ~>> (fn x -> {:ok, x * 2} end).()\n    {:ok, 20}\n\n    iex> import Loe\n    ...> :error ~>> (fn x -> x + 1 end).()\n    {:error, :error}","ref":"Loe.html#~%3E%3E/2-examples"},{"type":"extras","title":"Loe","doc":"# Loe\n\n[![License: MIT](https://img.shields.io/badge/license-MIT-blue.svg)](LICENSE.md)\n[![Hex.pm](https://img.shields.io/hexpm/v/loe.svg)](https://hex.pm/packages/loe)\n[![Documentation](https://img.shields.io/badge/documentation-gray)](https://hexdocs.pm/loe/)\n\n**Loe** is a tiny Elixir library for working with values that may be raw (`val`), wrapped as `{:ok, val}`, or wrapped as `{:error, reason}`.\n\nIt provides simple, composable tools to normalize, transform, and chain values using functions or infix macros — making it ideal for expressive, railway-style flows.","ref":"readme.html"},{"type":"extras","title":"Terminology - Loe","doc":"Throughout this library and documentation:\n\n- **Raw value** — A plain value not wrapped in a tuple (e.g., `42`)\n- **Success value** — A two-element tuple in the form `{:ok, value}`\n- **Error value** — A two-element tuple in the form `{:error, reason}`\n\nTuples like `{:ok, a, b}` or `{:error, a, b}` are not supported — they’ll be treated as raw values.","ref":"readme.html#terminology"},{"type":"extras","title":"Core Functions & Macros - Loe","doc":"### `lift/2` (or `~>>`)\n\nTransforms a raw or success value using a function.\n\n- Automatically wraps raw inputs as `{:ok, val}`\n- Skips transformation if the input is an error\n- Wraps raw results as `{:ok, val}` if the function returns a plain value\n\n```elixir\n4 ~>> (fn x -> x * 2 end).()        # => {:ok, 8}\n{:ok, 3} ~>> (fn x -> {:ok, x + 1} end).()   # => {:ok, 4}\n{:error, :fail} ~>> (fn _ -> :skip end).()   # => {:error, :fail}\n```\n\n### `tfil/2` (or `<~>`)\n\nTransforms an error value using a function.\n\n- Only applies when the input is an error\n- Returns a new `{:error, transformed}` tuple\n- Passes through success and raw values unchanged\n\n```elixir\n{:error, :bad} <~> fn e -> \"Reason: #{e}\" end\n# => {:error, \"Reason: bad\"}\n```\n\n### `unwrap!/1`\n\nUnwraps a success value or raises on error.\n\n```elixir\nLoe.unwrap!({:ok, 42})     # => 42\nLoe.unwrap!({:error, :fail}) # => ** (RuntimeError) Loe.unwrap!/1 encountered error: :fail\n```","ref":"readme.html#core-functions-macros"},{"type":"extras","title":"Installation - Loe","doc":"Add `:loe` to your `mix.exs` dependencies:\n\n```elixir\ndef deps do\n  [\n    {:loe, \"~> 0.1.0\"}\n  ]\nend\n```\n\nThen run:\n\n```bash\nmix deps.get\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Loe","doc":"Let’s say you have some simple input validation logic:\n\n```elixir\ndefmodule Validation do\n  def integer(v) when is_integer(v), do: {:ok, v}\n  def integer(_), do: {:error, :not_integer}\n\n  def positive(v) when v > 0, do: {:ok, v}\n  def positive(_), do: {:error, :not_positive}\nend\n\ndefmodule Data do\n  def double(v), do: 2 * v\nend\n\ndefmodule Error do\n  def transform(reason), do: %{message: \"Validation failed: #{reason}\"}\nend\n```\n\nNow import Loe to enable its macros:\n\n```elixir\nimport Loe\n```\n\nAnd chain validations like this:\n\n```elixir\n4\n~>> Validation.integer()\n~>> Validation.positive()\n# => {:ok, 4}\n\n-4\n~>> Validation.integer()\n~>> Validation.positive()\n# => {:error, :not_positive}\n\n3.4\n~>> Validation.integer()\n~>> Validation.positive()\n<~> Error.transform()\n# => {:error, %{message: \"Validation failed: not_integer\"}}\n```\n\n`Validation` functions return wrapped results, while `Data.double/1` returns a raw value.\nThanks to `lift/2`, Loe handles both seamlessly — so you can chain functions without worrying about return formats:\n\n```elixir\n4\n~>> Validation.integer()\n~>> Validation.positive()\n~>> Data.double()\n<~> Error.transform()\n# => {:ok, 8}\n```","ref":"readme.html#usage"},{"type":"extras","title":"Anonymous functions work too - Loe","doc":"```elixir\n2\n~>> (fn x -> x * 3 end).()\n~>> (fn x -> {:ok, x + 1} end).()\n<~> (fn e -> {:error, {:wrapped, e}} end).()\n# => {:ok, 7}\n```","ref":"readme.html#anonymous-functions-work-too"},{"type":"extras","title":"Works with error values directly - Loe","doc":"```elixir\n{:error, :oops}\n<~> (fn e -> \"Got: #{e}\" end).()\n# => {:error, \"Got: oops\"}\n```","ref":"readme.html#works-with-error-values-directly"},{"type":"extras","title":"Unwrapping - Loe","doc":"```elixir\n{:ok, 42} |> Loe.unwrap!()\n# => 42\n\n{:error, :fail} |> Loe.unwrap!()\n# => ** (RuntimeError) Loe.unwrap!/1 encountered error: :fail\n```","ref":"readme.html#unwrapping"},{"type":"extras","title":"Accepts raw or success values - Loe","doc":"```elixir\n# Both forms produce the same result:\n4\n~>> Validation.integer()\n~>> Validation.positive()\n\n{:ok, 4}\n~>> Validation.integer()\n~>> Validation.positive()\n```","ref":"readme.html#accepts-raw-or-success-values"},{"type":"extras","title":"Documentation - Loe","doc":"Full API docs available on [HexDocs](https://hexdocs.pm/loe).","ref":"readme.html#documentation"},{"type":"extras","title":"Contributing - Loe","doc":"Contributions are welcome via issues or pull requests.\nCreated and maintained by [Centib](https://github.com/Centib).","ref":"readme.html#contributing"},{"type":"extras","title":"License - Loe","doc":"Released under the [MIT License](LICENSE.md).","ref":"readme.html#license"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\n## [0.1.0] – 2025-07-11\n\nInitial release of **Loe**, a tiny Elixir library for chaining and transforming raw, `{:ok, _}`, and `{:error, _}` values.","ref":"changelog.html"},{"type":"extras","title":"Added - Changelog","doc":"- `lift/2` — Applies a function to raw or success values.\n- `tfil/2` — Applies a function to error values only.\n- `unwrap!/1` — Extracts the `{:ok, value}` or raises on error.\n- `~>>` — Infix macro for chaining with `lift/2`.\n- `<~>` — Infix macro for chaining with `tfil/2`.","ref":"changelog.html#added"},{"type":"extras","title":"LICENSE","doc":"MIT License\n\nCopyright (c) 2025 Centib\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}